/**************************************************************************
  2.00b thermometer with WiFi (prototype version)
 **************************************************************************/

#include <Adafruit_MLX90614.h> // thermometer library
#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7789.h> // Hardware-specific library for ST7789
#include <Fonts/FreeSans9pt7b.h> // for sans-serif font
#include <SdFat.h>                // SD card & FAT filesystem library
#include <Adafruit_SPIFlash.h>    // SPI / QSPI flash library
#include <Adafruit_ImageReader.h> // Image-reading functions
#include <SPI.h>                  // WiFi
#include <WiFiNINA.h>             // WiFi
#include <ArduinoHttpClient.h>    // Sending HTTP data
#include <ArduinoJson.h>          // JSON formatted data

#include "arduino_secrets.h" 
///////please enter your sensitive data in the Secret tab/arduino_secrets.h
char ssid[] = SECRET_SSID;        // your network SSID (name)
char pass[] = SECRET_PASS;    // your network password (use for WPA)
char api_key[] = API_KEY;    // your API key for sending temperature data to 2.00b

// for TFT display
#define USE_SD_CARD
#define TFT_CS        10
#define TFT_RST        9 // Or set to -1 and connect to Arduino RESET pin
#define TFT_DC         8
#define SD_CS          7 // SD card select pin

#if defined(USE_SD_CARD)
  SdFat                SD;         // SD card filesystem
  Adafruit_ImageReader reader(SD); // Image-reader object, pass in SD filesys
#endif

// thermometer
Adafruit_MLX90614 mlx = Adafruit_MLX90614();
const double adjDelta = 5.77; // compensation amount for body temperature, in F
const double INITIAL_TEMP = 0.0;
double adjTemp = INITIAL_TEMP;         // current measured and adjusted temperature, in F

// display
Adafruit_ST7789 tft = Adafruit_ST7789(TFT_CS, TFT_DC, TFT_RST);
Adafruit_Image       img;        // An image loaded into RAM
int32_t              width  = 0, // BMP image dimensions
                     height = 0;

const char* GOOD_STATUS = "GOOD TO GO";
char* logoFilename = "200blogo.bmp";
char* badStatusFilename = "bad.bmp";
char* goodStatusFilename = "good.bmp";
String NO_TEMP_YET = String("TAKE TEMP");
String greetingMsg = "Hello, 2.00b!";

const int buttonPin = 2; // button connected to D2
int currentButtonState = HIGH;
int lastButtonState = HIGH;
int buttonReadInterval = 50;
unsigned long lastDebounceTime = 0;  // the last time the output pin was toggled
unsigned long debounceDelay = 50;    // the debounce time; increase if the output 

// for double press detection
long buttonPressStartTime = 0;
long longPressTime = 500; // hold down for 1/2 second is a long press
boolean inButtonPress = false;
boolean inLongButtonPress = false;

int status = WL_IDLE_STATUS;
char server[] = "200b.mit.edu";
int port = 443; // 443 for HTTPS
String path = "/apps/health/add_temp/";
WiFiSSLClient wificlient;
HttpClient httpclient = HttpClient(wificlient, server, port);
const char* contentType = "application/json";

void setup() {
  //Initialize serial and wait for port to open:
  Serial.begin(9600);
  // If you want to see all the Serial messages,
  // open the Serial Monitor before starting the program.
  // If you don't see all the messages, you can make the program
  // wait until the Serial Monitor is ready before starting
  // by uncommenting the next line (remove "//" at the beginning)
  while (!Serial) ; // wait for Serial to be ready before going on
  // line above needs to be commented out to work without being connected to computer!
  
  Serial.println("Starting thermometer with wifi");

  pinMode(buttonPin, INPUT_PULLUP); // set button to HIGH
  mlx.begin();        // start temperature sensor
  
  // initialize and setup display
  tft.init(135, 240); // Init ST7789 1.14" 135x240 TFT
  tft.fillScreen(ST77XX_BLACK);
  tft.setTextColor(ST77XX_WHITE);
  tft.setFont(&FreeSans9pt7b);
  tft.setTextSize(2);
  
#if defined(USE_SD_CARD)
  if(!SD.begin(SD_CS, SD_SCK_MHZ(10))) {
    Serial.println("Problem with SD card...halting");
    for(;;); // Fatal error, do not continue
  }
#endif
  
  showLogoAndGreeting();
  Serial.println("Press button to take temperature. Long press to send temperature.");
}

void loop() {
  int newButtonState = digitalRead(buttonPin);

  if (newButtonState != lastButtonState) {
    lastDebounceTime = millis();
  }

  if ((millis() - lastDebounceTime) > debounceDelay) {
    if (newButtonState != currentButtonState) { // button state has changed
      currentButtonState = newButtonState;

      if (currentButtonState == LOW) { // button pressed
        if (inButtonPress == false) {
          inButtonPress = true;
          buttonPressStartTime = millis();
        }
      } else { // button released
        if (inButtonPress == true) { // be sure we were in a button press
          if (millis() - buttonPressStartTime > longPressTime) { // long button press
            // do long button press activity
            Serial.println("Long button press detected");
            doLongButtonPressAction();
          } else { // short button press
            // do short button press activity
            Serial.println("Short button press detected");
            doShortButtonPressAction();
          }
          inButtonPress = false;
        }
      } 
    }
  }

  // save the reading. Next time through the loop, it'll be the lastButtonState:
  lastButtonState = newButtonState;
}

void doShortButtonPressAction() {
  // average 10 readings over a second, then display that reading
  double totalTemp = 0.0;
  for (int i=0; i<10; i++) {
    double newTemp = mlx.readObjectTempF();
    Serial.print(newTemp); Serial.print("   ");
    totalTemp = totalTemp + newTemp;
    delay(100);
  }
  Serial.println();
  double objTemp = totalTemp/10.0;
  Serial.print("Measured = "); Serial.print(objTemp); Serial.println("*F");
  adjTemp = objTemp + adjDelta; // set value of adjTemp globally
  Serial.print("Adjusted = "); Serial.print(adjTemp); Serial.println("*F");
  displayTemp(adjTemp);
}

void doLongButtonPressAction() {
  if (adjTemp == INITIAL_TEMP) { // no temperature readings yet
    displayMsg(NO_TEMP_YET);
  } else { // send temperature reading
    String result = sendTemp(adjTemp);
    if (result == GOOD_STATUS) {
      loadFullScreenImageFile(goodStatusFilename);
    } else {
      loadFullScreenImageFile(badStatusFilename);
      delay(2000);
      displayMsg(result);
    }
  }
}

void showLogoAndGreeting() {
  loadFullScreenImageFile(logoFilename);
  delay(1500); // wait 1.5 seconds for people to see the logo

  displayMsg(greetingMsg);
}

void displayTemp(double temp) {
  String msg = String(temp)+"*F";
  displayMsg(msg);
}

void loadFullScreenImageFile(char* bmp_file) {
  // Load full-screen BMP file at position (0,0) (top left).
  tft.setRotation(1); // orientation for final build
  Serial.print("Loading ");
  Serial.print(bmp_file);
  Serial.print("... ");
  ImageReturnCode stat = reader.drawBMP(bmp_file, tft, 0, 0);
  reader.printStatus(stat);   // How'd we do? - message to Serial Monitor
}

void displayMsg(String &msg) {
  tft.setRotation(3); // bug in GFX library does not fully fill screen in orientation 1
  tft.fillScreen(ST77XX_BLACK);
  tft.setRotation(1); // orientation for final build
  int16_t x1, y1;
  uint16_t w, h;
  tft.getTextBounds(msg, 0, 80, &x1, &y1, &w, &h); // find width of text
  tft.setCursor((240-w)/2.0,80); // center text on screen
  tft.print(msg);
}

void displayWiFiMsg(char* network_name, int count) {
  tft.setRotation(3); // bug in GFX library does not fully fill screen in orientation 1
  tft.fillScreen(ST77XX_BLACK);
  tft.setRotation(1); // orientation for final build
  tft.setCursor(0,30);
  tft.println("Connecting to");
  tft.println(network_name);
  if (count > 0) {
    tft.println("...again");
  }
}

void setupWiFi() {
  // check for the WiFi module:
  if (WiFi.status() == WL_NO_MODULE) {
    Serial.println("Communication with WiFi module failed - halting!");
    // don't continue
    while (true);
  }

  // attempt to connect to WiFi network:
  int count = 0;
  while (status != WL_CONNECTED) {
    Serial.print("\nAttempting to connect to SSID: ");
    Serial.print(ssid);
    displayWiFiMsg(ssid, count);
    status = WiFi.begin(ssid, pass);
    delay(1000);
    count = count+1;
  }
  Serial.println("...connected!");
  tft.println("connected!");
}

String sendTemp(double temp) {
  if (WiFi.status() != WL_CONNECTED) {
    setupWiFi();
  }
  Serial.print("\nSending temperature ");
  Serial.print(temp);
  Serial.print(" to ");
  Serial.print(server);
  Serial.print(":");
  Serial.print(port);
  Serial.println(path);
  
  // constructing data structure for sending over WiFi
  DynamicJsonDocument jsonDoc(50);
  jsonDoc["api_key"] = api_key;
  jsonDoc["temp"] = temp;
  String jsonData;
  serializeJson(jsonDoc, jsonData);
  Serial.print("JSON data: ");
  Serial.println(jsonData);

  httpclient.post(path, contentType, jsonData);
  int statusCode = httpclient.responseStatusCode();
  String response = httpclient.responseBody();

  Serial.print("Status code: ");
  Serial.println(statusCode);
  Serial.print("Response: ");
  Serial.println(response);
  if (statusCode == 200) {
    return response;
  } else {
    return "???";
  }
}
